@@ -51,8 +51,8 @@
 #include "dlg-color-balance.h"
 #include "dlg-preferences.h"
 #include "dlg-rename-series.h"
-#include "dlg-save-image.h"
 #include "dlg-scale-image.h"
+#include "dlg-crop.h"
 #include "fullscreen.h"
 #include "gconf-utils.h"
 #include "gth-pixbuf-op.h"
@@ -76,36 +76,21 @@ typedef enum {
 	WALLPAPER_ALIGN_TILED     = 0,
 	WALLPAPER_ALIGN_CENTERED  = 1,
 	WALLPAPER_ALIGN_STRETCHED = 2,
-	WALLPAPER_ALIGN_SCALED    = 3
+	WALLPAPER_ALIGN_SCALED    = 3,
+	WALLPAPER_NONE            = 4
 } WallpaperAlign;
 
 
-void 
-file_new_window_command_impl (BonoboUIComponent *uic, 
-			      gpointer           user_data, 
-			      const char        *verbname)
+static void
+open_new_window_at_location (GThumbWindow *window,
+			     const char   *location)
 {
-	GThumbWindow *window = user_data;
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 	GThumbWindow *new_window;
-	char         *location = NULL;
 	int           width, height;
 
-	if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) 
-	    && (window->dir_list->path != NULL))
-		location = g_strconcat ("file://",
-					window->dir_list->path,
-					NULL);
-	else if ((window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST) 
-		 && (window->catalog_path != NULL))
-		location = g_strconcat ("catalog://",
-					window->catalog_path,
-					NULL);	
-
-	if (location != NULL) {
+	if (location != NULL) 
 		preferences_set_startup_location (location);
-		g_free (location);
-	}
 
 	/* Save visualization options. */
 
@@ -133,6 +118,30 @@ file_new_window_command_impl (BonoboUIComponent *uic,
 }
 
 
+void 
+file_new_window_command_impl (BonoboUIComponent *uic, 
+			      gpointer           user_data, 
+			      const char        *verbname)
+{
+	GThumbWindow *window = user_data;
+	char         *location = NULL;
+
+	if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) 
+	    && (window->dir_list->path != NULL))
+		location = g_strconcat ("file://",
+					window->dir_list->path,
+					NULL);
+	else if ((window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST) 
+		 && (window->catalog_path != NULL))
+		location = g_strconcat ("catalog://",
+					window->catalog_path,
+					NULL);	
+
+	open_new_window_at_location (window, location);
+	g_free (location);
+}
+
+
 void 
 file_close_window_command_impl (BonoboUIComponent *uic, 
 				gpointer           user_data, 
@@ -178,39 +187,15 @@ image_open_with_command_impl (BonoboUIComponent *uic,
 }
 
 
-static void
-image_saved_func (char     *filename,
-		  gpointer  data)
-{
-	GThumbWindow *window = data;
-	if (filename != NULL) 
-		window_load_image (window, filename);
-}
-
-
 void 
 file_save_command_impl (BonoboUIComponent *uic, 
 			gpointer           user_data, 
 			const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
-	char         *current_folder = NULL;
+	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 
-	if (window->image_path != NULL)
-		current_folder = g_strdup (window->image_path);
-
-	else if (window->dir_list->path != NULL)
-		current_folder = g_strconcat (window->dir_list->path,
-					      "/", 
-					      NULL);
-
-	dlg_save_image (GTK_WINDOW (window->app), 
-			current_folder,
-			image_viewer_get_current_pixbuf (IMAGE_VIEWER (window->viewer)),
-			image_saved_func,
-			window);
-
-	g_free (current_folder);
+	window_save_pixbuf (window, image_viewer_get_current_pixbuf (viewer));
 }
 
 
@@ -230,16 +215,29 @@ file_print_command_impl (BonoboUIComponent *uic,
 			 const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
-	char         *uri;
+	GList        *list;
+	
+	list = gth_file_view_get_file_list_selection (window->file_list->view);
+	if (list != NULL)
+		print_catalog_dlg (GTK_WINDOW (window->app), list);
+	path_list_free (list);
+}
 
-	if (window->image_path == NULL)
-		return;
 
-	uri = g_strconcat ("file://", window->image_path, NULL);
-	print_image_dlg (GTK_WINDOW (window->app), 
-			 IMAGE_VIEWER (window->viewer),
-			 uri);
-	g_free (uri);
+void 
+file_camera_import_command_impl (BonoboUIComponent *uic, 
+				 gpointer           user_data, 
+				 const gchar       *verbname)
+{
+#ifdef HAVE_LIBGPHOTO
+
+	GThumbWindow *window = user_data;
+	void (*module) (GThumbWindow *window);
+
+	if (gthumb_module_get ("dlg_photo_importer", (gpointer*) &module))
+		(*module) (window);
+
+#endif /*HAVE_LIBGPHOTO */
 }
 
 
@@ -433,9 +431,19 @@ duplicate_file (GThumbWindow *window,
 	g_free (old_name_no_ext);
 
 	if (file_copy (old_path, new_path)) {
+		GList *file_list;
+
 		cache_copy (old_path, new_path);
 		comment_copy (old_path, new_path);
 
+		all_windows_remove_monitor ();
+
+		file_list = g_list_prepend (NULL, new_path);
+		all_windows_notify_files_created (file_list);
+		g_list_free (file_list);
+
+		all_windows_add_monitor ();
+
 	} else {
 		char      *utf8_path;
 		char      *msg;
@@ -463,7 +471,6 @@ duplicate_file (GThumbWindow *window,
 			g_free (new_name);
 			return;
 		}
-					   
 	}
 
 	g_free (new_name);
@@ -837,26 +844,56 @@ edit_remove_from_catalog_command_impl (BonoboUIComponent *uic,
 
 
 static void
-folder_open (GThumbWindow *window,
-	     const char   *viewer,
-	     const char   *path,
-	     const char   *working_dir) 
+show_folder (GThumbWindow *window,
+	     const char   *path)
 {
 	GError *err = NULL;
-	char   *argv[5];
+	char   *uri;
 
-	argv[0] = (char*) viewer;
-	argv[1] = (char*) path;
-	argv[2] = NULL;
-	
-	if (! g_spawn_sync (working_dir, 
-			    argv, NULL, 
-			    G_SPAWN_SEARCH_PATH, 
-			    NULL, NULL, 
-			    NULL, NULL,
-			    NULL, &err))
+	uri = g_strconcat ("file://", path, NULL);
+	if (! gnome_url_show (uri, &err))
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window->app),
 						   &err);
+	g_free (uri);
+}
+
+
+void 
+edit_folder_view_command_impl (BonoboUIComponent *uic, 
+			       gpointer           user_data, 
+			       const gchar       *verbname)
+{
+	GThumbWindow  *window = user_data;
+	char          *path;
+
+	path = dir_list_get_selected_path (window->dir_list);
+	if (path == NULL) 
+		return;
+
+	window_go_to_directory (window, path);
+
+	g_free (path);
+}
+
+
+void 
+edit_folder_view_new_window_command_impl (BonoboUIComponent *uic, 
+					  gpointer           user_data, 
+					  const gchar       *verbname)
+{
+	GThumbWindow  *window = user_data;
+	char          *path;
+	char          *location;
+
+	path = dir_list_get_selected_path (window->dir_list);
+	if (path == NULL) 
+		return;
+
+	location = g_strconcat ("file://", path, NULL);
+	open_new_window_at_location (window, location);
+	g_free (location);
+
+	g_free (path);
 }
 
 
@@ -872,8 +909,7 @@ edit_folder_open_nautilus_command_impl (BonoboUIComponent *uic,
 	if (path == NULL) 
 		return;
 
-	folder_open (window, "nautilus", path, NULL);
-
+	show_folder (window, path);
 	g_free (path);
 }
 
@@ -988,7 +1024,7 @@ edit_current_folder_open_nautilus_command_impl (BonoboUIComponent *uic,
 	path = window->dir_list->path;
 	if (path == NULL) 
 		return;
-	folder_open (window, "nautilus", path, NULL);
+	show_folder (window, path);
 }
 
 
@@ -1059,34 +1095,47 @@ folder_rename (GThumbWindow *window,
 				       _("The name \"%s\" is already used. " "Please use a different name."), utf8_name);
 		g_free (utf8_name);
 
-	} else if (rename (old_path, new_path) == 0) { 
-		char *old_cache_path, *new_cache_path;
-
-		/* Update cache info. */
-
-		/* Comment cache. */
-
+	} else {
+		char *old_cache_path = NULL;
+		char *old_folder_comment = NULL;
+		
 		old_cache_path = comments_get_comment_dir (old_path, TRUE, TRUE);
 		if (! path_is_dir (old_cache_path)) {
 			g_free (old_cache_path);
 			old_cache_path = comments_get_comment_dir (old_path, FALSE, TRUE);
 		}
-		new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
-		rename (old_cache_path, new_cache_path);
+
+		old_folder_comment = comments_get_comment_filename (old_path, TRUE, TRUE);
+
+		if (rename (old_path, new_path) == 0) { 
+			char *new_cache_path;
+			char *new_folder_comment;
+
+			/* Comment cache. */
+
+			new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
+			new_folder_comment = comments_get_comment_filename (new_path, TRUE, TRUE);
+
+			rename (old_cache_path, new_cache_path);
+			rename (old_folder_comment, new_folder_comment);
+
+			g_free (new_cache_path);
+			g_free (new_folder_comment);
+			
+			all_windows_notify_directory_rename (old_path, new_path);
+
+		} else {
+			char *utf8_path;
+			utf8_path = g_filename_to_utf8 (old_path, -1, NULL, NULL, NULL);
+			_gtk_error_dialog_run (GTK_WINDOW (window->app),
+					       _("Could not rename the folder \"%s\": %s"),
+					       utf8_path,
+					       errno_to_string ());
+			g_free (utf8_path);
+		}
+
 		g_free (old_cache_path);
-		g_free (new_cache_path);
-
-		all_windows_notify_directory_rename (old_path, new_path);
-
-	} else {
-		char *utf8_path;
-
-		utf8_path = g_filename_to_utf8 (old_path, -1, NULL, NULL, NULL);
-		_gtk_error_dialog_run (GTK_WINDOW (window->app),
-                                       _("Could not rename the folder \"%s\": %s"),
-                                       utf8_path,
-                                       errno_to_string ());
-		g_free (utf8_path);
+		g_free (old_folder_comment);
 	}
 
 	all_windows_add_monitor ();
@@ -1389,45 +1438,60 @@ folder_copy__response_cb (GObject *object,
 				 folder_copy__continue, 
 				 g_strdup (old_path));
 
-	} else if (rename (old_path, new_path) == 0) { 
-		char *old_cache_path, *new_cache_path;
-
-		/* moving folders on the same file system can be implemeted 
-		 * with rename, which is faster. */
+	} else {
+		char *old_cache_path = NULL;
+		char *old_folder_comment = NULL;
 
 		/* Comment cache. */
-
+			
 		old_cache_path = comments_get_comment_dir (old_path, TRUE, TRUE);
 		if (! path_is_dir (old_cache_path)) {
 			g_free (old_cache_path);
 			old_cache_path = comments_get_comment_dir (old_path, FALSE, TRUE);
 		}
-		new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
 
-		if (path_is_dir (old_cache_path)) {
-			char *parent_dir;
+		old_folder_comment= comments_get_comment_filename (old_path, TRUE, TRUE);
 
-			parent_dir = remove_level_from_path (new_cache_path);
-			ensure_dir_exists (parent_dir, 0755);
-			g_free (parent_dir);
+		if (rename (old_path, new_path) == 0) { 
+			char *new_cache_path;
+			char *new_folder_comment;
 
-			rename (old_cache_path, new_cache_path);
+			/* moving folders on the same file system can be implemeted 
+			 * with rename, which is faster. */
+
+			new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
+			new_folder_comment = comments_get_comment_filename (new_path, TRUE, TRUE);
+
+			if (path_is_dir (old_cache_path)) {
+				char *parent_dir;
+				
+				parent_dir = remove_level_from_path (new_cache_path);
+				ensure_dir_exists (parent_dir, 0755);
+				g_free (parent_dir);
+				
+				rename (old_cache_path, new_cache_path);
+			}
+
+			rename (old_folder_comment, new_folder_comment);
+
+			g_free (new_cache_path);
+			g_free (new_folder_comment);
+
+			all_windows_notify_directory_rename (old_path, new_path);
+
+		} else {
+			char *utf8_path;
+
+			utf8_path = g_filename_to_utf8 (old_path, -1, NULL, NULL, NULL);
+			_gtk_error_dialog_run (GTK_WINDOW (window->app),
+					       message,
+					       utf8_path,
+					       errno_to_string ());
+			g_free (utf8_path);
 		}
 
 		g_free (old_cache_path);
-		g_free (new_cache_path);
-
-		all_windows_notify_directory_rename (old_path, new_path);
-
-	} else {
-		char *utf8_path;
-
-		utf8_path = g_filename_to_utf8 (old_path, -1, NULL, NULL, NULL);
-		_gtk_error_dialog_run (GTK_WINDOW (window->app),
-				       message,
-                                       utf8_path,
-                                       errno_to_string ());
-		g_free (utf8_path);
+		g_free (old_folder_comment);
 	}
 
 	g_free (dest_dir);
@@ -1520,6 +1584,110 @@ edit_current_folder_move_command_impl (BonoboUIComponent *uic,
 }
 
 
+/**/
+
+typedef struct {
+	GThumbWindow  *window;
+	GList         *file_list;
+	GList         *add_list, *remove_list;
+} FolderCategoriesData;
+
+
+static void
+edit_current_folder_categories__done (gpointer data)
+{
+	FolderCategoriesData *fcdata = data;
+	GList                *scan;
+	
+	for (scan = fcdata->file_list; scan; scan = scan->next) {
+		const char  *filename = scan->data;
+		CommentData *cdata;
+		GList       *scan2;
+
+		cdata = comments_load_comment (filename);
+		if (cdata == NULL)
+			cdata = comment_data_new ();
+		else 
+			for (scan2 = fcdata->remove_list; scan2; scan2 = scan2->next) {
+				const char *k = scan2->data;
+				comment_data_remove_keyword (cdata, k);
+			}
+
+		for (scan2 = fcdata->add_list; scan2; scan2 = scan2->next) {
+			const char *k = scan2->data;
+			comment_data_add_keyword (cdata, k);
+		}
+
+		comments_save_categories (filename, cdata);
+		comment_data_free (cdata);
+
+		dir_list_remove_directory (fcdata->window->dir_list, filename);
+		dir_list_add_directory (fcdata->window->dir_list, filename);
+	}
+
+	path_list_free (fcdata->file_list);
+	path_list_free (fcdata->add_list);
+	path_list_free (fcdata->remove_list);
+	g_free (fcdata);
+}
+
+
+static void 
+edit_current_folder_categories (GThumbWindow  *window,
+				const char    *path)
+{
+	FolderCategoriesData *fcdata;
+
+	fcdata = g_new0 (FolderCategoriesData, 1);
+	fcdata->window = window;
+	fcdata->add_list = NULL;
+	fcdata->remove_list = NULL;
+	fcdata->file_list = g_list_prepend (NULL, g_strdup (path));
+
+	dlg_choose_categories (GTK_WINDOW (window->app),
+			       fcdata->file_list,
+			       NULL,
+			       &(fcdata->add_list),
+			       &(fcdata->remove_list),
+			       edit_current_folder_categories__done,
+			       fcdata);
+}
+
+
+void 
+edit_current_folder_categories_command_impl (BonoboUIComponent *uic, 
+					     gpointer           user_data, 
+					     const gchar       *verbname)
+{
+	GThumbWindow         *window = user_data;
+	char                 *path;
+
+	path = window->dir_list->path;
+	if (path == NULL)
+		return;
+
+	edit_current_folder_categories (window, path);
+}
+
+
+void 
+edit_folder_categories_command_impl (BonoboUIComponent *uic, 
+				     gpointer           user_data, 
+				     const gchar       *verbname)
+{
+	GThumbWindow *window = user_data;
+	char         *path;
+
+	path = dir_list_get_selected_path (window->dir_list);
+	if (path == NULL)
+		return;
+
+	edit_current_folder_categories (window, path);
+
+	g_free (path);
+}
+
+
 void 
 edit_folder_copy_command_impl (BonoboUIComponent *uic, 
 			       gpointer           user_data, 
@@ -1943,9 +2111,9 @@ edit_current_catalog_redo_search_command_impl (BonoboUIComponent *uic,
 
 
 void 
-alter_image_rotate_command_impl (BonoboUIComponent *uic, 
-				 gpointer           user_data, 
-				 const gchar       *verbname)
+alter_image_rotate_90_command_impl (BonoboUIComponent *uic, 
+				    gpointer           user_data, 
+				    const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
@@ -1962,9 +2130,9 @@ alter_image_rotate_command_impl (BonoboUIComponent *uic,
 
 
 void 
-alter_image_rotate_cc_command_impl (BonoboUIComponent *uic, 
-				    gpointer           user_data, 
-				    const gchar       *verbname)
+alter_image_rotate_90_cc_command_impl (BonoboUIComponent *uic, 
+				       gpointer           user_data, 
+				       const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
@@ -2194,6 +2362,16 @@ alter_image_scale_command_impl (BonoboUIComponent *uic,
 }
 
 
+void 
+alter_image_crop_command_impl (BonoboUIComponent *uic, 
+			       gpointer           user_data, 
+			       const gchar       *verbname)
+{
+	GThumbWindow *window = user_data;
+	dlg_crop (window);
+}
+
+
 void 
 view_zoom_in_command_impl (BonoboUIComponent *uic, 
 			   gpointer           user_data, 
@@ -2472,28 +2650,35 @@ set_as_wallpaper (GThumbWindow   *window,
 	GConfClient *client;
 	char        *options;
 
-	if ((image_path == NULL) || ! path_is_file (image_path))
-		return;
-
 	client = gconf_client_get_default ();
-	gconf_client_set_string (client, 
-				 "/desktop/gnome/background/picture_filename",
-				 image_path,
-				 NULL);
-	switch (align) {
-	case WALLPAPER_ALIGN_TILED:
-		options = "wallpaper";
-		break;
-	case WALLPAPER_ALIGN_CENTERED:
-		options = "centered";
-		break;
-	case WALLPAPER_ALIGN_STRETCHED:
-		options = "stretched";
-		break;
-	case WALLPAPER_ALIGN_SCALED:
-		options = "scaled";
-		break;
+
+	if ((image_path == NULL) || ! path_is_file (image_path)) 
+		options = "none";
+
+	else {
+		gconf_client_set_string (client, 
+					 "/desktop/gnome/background/picture_filename",
+					 image_path,
+					 NULL);
+		switch (align) {
+		case WALLPAPER_ALIGN_TILED:
+			options = "wallpaper";
+			break;
+		case WALLPAPER_ALIGN_CENTERED:
+			options = "centered";
+			break;
+		case WALLPAPER_ALIGN_STRETCHED:
+			options = "stretched";
+			break;
+		case WALLPAPER_ALIGN_SCALED:
+			options = "scaled";
+			break;
+		case WALLPAPER_NONE:
+			options = "none";
+			break;
+		}
 	}
+
 	gconf_client_set_string (client, 
 				 "/desktop/gnome/background/picture_options", 
 				 options,
@@ -2558,6 +2743,8 @@ wallpaper_restore_command_impl (BonoboUIComponent *uic,
 		align_type = WALLPAPER_ALIGN_STRETCHED;
 	else if (strcmp (preferences.wallpaperAlign, "scaled") == 0)
 		align_type = WALLPAPER_ALIGN_SCALED;
+	else if (strcmp (preferences.wallpaperAlign, "none") == 0)
+		align_type = WALLPAPER_NONE;
 
 	set_as_wallpaper (window, 
 			  preferences.wallpaper,
