@@ -253,7 +253,7 @@ help_cb (GtkWidget  *widget,
 	GError *err;
 
 	err = NULL;  
-	gnome_help_display ("gthumb", "comments", &err);
+	gnome_help_display ("gthumb", "gthumb-comments", &err);
 	
 	if (err != NULL) {
 		GtkWidget *dialog;
@@ -396,14 +396,8 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 	data->have_exif_data = FALSE;
 #else
 	{
-		ExifData *edata;
-		char     *first_image = data->file_list->data;
-
-		edata = exif_data_new_from_file (first_image);
-		data->have_exif_data = edata != NULL;
-
-		if (edata != NULL) 
-			exif_data_unref (edata);
+		char *first_image = data->file_list->data;
+		data->have_exif_data = have_exif_data (first_image);
 	}
 #endif
 
@@ -505,8 +499,14 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 
 	gtk_widget_grab_focus (data->note_text_view);
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
-				      GTK_WINDOW (window->app));
-	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
+	{
+		GtkWidget *parent_win = window->viewer;
+		while (parent_win != NULL && !GTK_IS_WINDOW(parent_win))
+			parent_win = parent_win->parent;
+		if (parent_win != NULL)
+			gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (parent_win));
+	}
+
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
 	gtk_widget_show (data->dialog);
 }
