@@ -43,6 +43,10 @@
 #include "preferences.h"
 #include "icons/pixbufs.h"
 #include "typedefs.h"
+#include "comments.h"
+#include "pixbuf-utils.h"
+#include "commands-impl.h"
+
 
 #define ICON_NAME_DIRECTORY "gnome-fs-directory"
 
@@ -51,14 +55,14 @@ GList          *window_list = NULL;
 FullScreen     *fullscreen;
 char          **file_urls, **dir_urls;
 int             n_file_urls, n_dir_urls;
-int             StartInFullscreen;
-int             StartSlideshow;
+int             StartInFullscreen = FALSE;
+int             StartSlideshow = FALSE;
 int             ViewFirstImage = FALSE;
 int             HideSidebar = FALSE;
 gboolean        ExitAll = FALSE;
 char           *ImageToDisplay = NULL;
 gboolean        FirstStart = TRUE;
-
+gboolean        ImportPhotos = FALSE;
 
 static gboolean        view_comline_catalog = FALSE;
 static gboolean        view_single_image = FALSE;
@@ -77,14 +81,18 @@ static gboolean load_session        (void);
 
 
 struct poptOption options[] = {
-	{ "fullscreen", 'f', POPT_ARG_NONE, &StartInFullscreen, 0,
+	{ "fullscreen", '\0', POPT_ARG_NONE, &StartInFullscreen, 0,
 	  N_("Start in fullscreen mode"),
 	  0 },
 
-	{ "slideshow", 's', POPT_ARG_NONE, &StartSlideshow, 0,
+	{ "slideshow", '\0', POPT_ARG_NONE, &StartSlideshow, 0,
 	  N_("Automatically start a slideshow"),
 	  0 },
 
+	{ "import-photos", '\0', POPT_ARG_NONE, &ImportPhotos, 0,
+	  N_("Automatically import digital camera photos"),
+	  0 },
+
 	{ NULL, '\0', 0, NULL, 0 }
 };
 
@@ -95,14 +103,18 @@ struct poptOption options[] = {
 static gboolean 
 check_whether_to_set_fullscreen (gpointer data) 
 {
-	if (first_window == NULL)
-		return FALSE;
-
-	if (StartInFullscreen) {
-		StartInFullscreen = FALSE;
+	if ((first_window != NULL) && StartInFullscreen) 
 		fullscreen_start (fullscreen, first_window);
-	}
-	
+	StartInFullscreen = FALSE;
+	return FALSE;
+}
+
+
+static gboolean 
+check_whether_to_import_photos (gpointer data) 
+{
+	if ((first_window != NULL) && ImportPhotos) 
+		file_camera_import_command_impl (NULL, first_window, NULL);		
 	return FALSE;
 }
 
@@ -137,17 +149,14 @@ main (int argc, char *argv[])
 			       GNOME_PARAM_POPT_CONTEXT,
 			       g_value_init (&value, G_TYPE_POINTER));
 	pctx = g_value_get_pointer (&value);
-
 	glade_gnome_init ();
-
 	gthumb_init ();
-
 	initialize_data (pctx);
 	poptFreeContext (pctx);
-
 	prepare_app ();
 
 	g_idle_add (check_whether_to_set_fullscreen, NULL);
+	g_idle_add (check_whether_to_import_photos, NULL);
 
 	bonobo_main ();
 
@@ -364,8 +373,11 @@ prepare_app ()
 	if (! view_comline_catalog 
 	    && (n_dir_urls == 0) 
 	    && (n_file_urls == 0)) {
+		if (ImportPhotos)
+			preferences_set_startup_location (NULL);
 		current_window = window_new ();
-		gtk_widget_show (current_window->app);
+		if (!ImportPhotos)
+			gtk_widget_show (current_window->app);
 		if (first_window == NULL)
 			first_window = current_window;
 	}
@@ -502,9 +514,11 @@ all_windows_notify_files_deleted (GList *list)
 	for (scan = window_list; scan; scan = scan->next) {
 		GThumbWindow *window = scan->data;
 
+		/* FIXME
 		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
 		    window->monitor_enabled)
 			continue;
+		*/
 
 		window_notify_files_deleted (window, list);
 	}
@@ -772,21 +786,19 @@ get_folder_pixbuf (double icon_size)
 	}
 	
 	if (folder_pixbuf != NULL) {
-		int       new_w, new_h;
-		int       w, h;
-		double    factor;
+		int w, h;
 		
 		w = gdk_pixbuf_get_width (folder_pixbuf);
 		h = gdk_pixbuf_get_height (folder_pixbuf);
-		
-		factor = MIN (icon_size / w, icon_size / h);
-		new_w  = MAX ((gint) (factor * w), 1);
-		new_h  = MAX ((gint) (factor * h), 1);
-				
-		pixbuf = gdk_pixbuf_scale_simple (folder_pixbuf,
-						  new_w,
-						  new_h,
-						  GDK_INTERP_BILINEAR);
+		if (scale_keepping_ratio (&w, &h, icon_size, icon_size)) 
+			pixbuf = gdk_pixbuf_scale_simple (folder_pixbuf,
+							  w,
+							  h,
+							  GDK_INTERP_BILINEAR);
+		else {
+			pixbuf = folder_pixbuf;
+			g_object_ref (pixbuf);
+		}
 	}
 
 	return pixbuf;
@@ -973,3 +985,26 @@ get_default_folder_pixbuf_size (GtkWidget *widget)
                                            &icon_width, &icon_height);
 	return MAX (icon_width, icon_height);
 }
+
+
+gboolean
+folder_is_film (const char *folder)
+{
+	CommentData *cdata;
+	gboolean     film = FALSE;
+
+	folder = pref_util_remove_prefix (folder);
+
+	cdata = comments_load_comment (folder);
+	if (cdata != NULL) {
+		int i;
+		for (i = 0; i < cdata->keywords_n; i++)
+			if (g_utf8_collate (cdata->keywords[i], _("Film")) == 0) {
+				film = TRUE;
+				break;
+			}
+		comment_data_free (cdata);
+	}
+
+	return film;
+}
